#include "skiller.h"
#include <memory.h>

skillerctl_packet_t* skillerctl_unpack(skillerctl_packet_t *packet, uint8_t *data, size_t length) {
    if (*(unsigned char*)data != 0x04 || length < 1) {
        return NULL;
    }

    packet->magic    = *(uint8_t*)(data);
    packet->checksum = *(uint16_t*)(data + 1);
    packet->command  = *(uint8_t*)(data + 3);
    packet->length   = *(uint8_t*)(data + 4);
    packet->address  = *(uint8_t*)(data + 5);
    // 1 null byte
    memcpy(packet->payload, data + 8, packet->length);

    return packet;
}

void* skillerctl_pack(skillerctl_packet_t *packet, uint8_t *data) {
    memcpy(data, &packet->magic, 1);
    memcpy(data + 1, &packet->checksum, 2);
    memcpy(data + 3, &packet->command, 1);
    memcpy(data + 4, &packet->length, 1);
    memcpy(data + 5, &packet->address, 2);
    // 1 null byte
    memcpy(data + 8, packet->payload, SK_MAX_PAYLOAD);

    return packet;
}

void skillerctl_make_packet(skillerctl_packet_t *packet, uint8_t command, uint16_t address, void* payload, uint8_t size)
{
    packet->magic   = SK_MAGIC;
    packet->address = address;
    packet->command = command;
    packet->length  = size;

    memset(packet->payload, 0, SK_MAX_PAYLOAD);

    if (payload && size) 
        memcpy(packet->payload, payload, size);

    packet->checksum = skillectl_packet_checksum(packet);
}

uint16_t skillectl_packet_checksum(skillerctl_packet_t *packet)
{
    uint16_t checksum = 0;
    
    checksum += (packet->command) & 0xFF;
    checksum += (packet->address) & 0xFF;
    checksum += (packet->address >> 8) & 0xFF;
    checksum += (packet->length) & 0xFF;

    for (int i = 0; i < packet->length; i++) {
        checksum += packet->payload[i];
    }

    return checksum;
}

void skillerctl_color_command(skillerctl_packet_t *packet, skillerctl_keyaddr_t address, uint32_t color)
{
    skillerctl_payload_color_t payload;

    payload.color[0].r = (color >> 16) & 0xFF;
    payload.color[0].g = (color >> 8)  & 0xFF;
    payload.color[0].b = color         & 0xFF;

    skillerctl_make_packet(packet, SK_CMD_COLOR, address, &payload, 3);
}

void skillerctl_enable_command(skillerctl_packet_t *packet)
{
    skillerctl_payload_color_t payload;
    skillerctl_make_packet(packet, SK_CMD_ENABLE, 0x0000, NULL, 0);
}

void skillerctl_disable_command(skillerctl_packet_t *packet)
{
    skillerctl_payload_color_t payload;
    skillerctl_make_packet(packet, SK_CMD_DISABLE, 0x0000, NULL, 0);
}
